//
//  SSView.m
//  SolSystem
//
//  Created by Paul Jungwirth on 11/14/09.
//  Copyright 2009 Illuminated Computing. All rights reserved.
//

#import "SSView.h"
#import "System.h"
#import "Planet.h"

@implementation SSView

- (id)initWithFrame:(NSRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        // Initialization code here.
		_speed = 900;
		_system = [[System alloc] init];
		
		float shorterSide = frame.size.width;
		if (shorterSide > frame.size.height) shorterSide = frame.size.height;
		NSLog(@"shorterSize = %f", shorterSide);
		
		_scale = [_system maxPos] * 1.2 * 2 / shorterSide; // *1.2 to give some margin, *2 to give both positive and negative values
		NSLog(@"maxPos = %f", [_system maxPos]);
		NSLog(@"km / pixels = %f", _scale);
		
		// TODO: If this were in a different thread, would it prevent the universe from locking up while the slider is moving?:
		//		If it were in a different thread, we'd need to lock _speed when we access it.
		//		In that case, I should lock it once at the beginning of advanceSystem, not each time through its for-loop!
		// [NSTimer scheduledTimerWithTimeInterval:0.001 target:self selector:@selector(advanceSystem) userInfo:nil repeats:YES];
		// TODO: need to create a memory pool?
		[NSThread detachNewThreadSelector:@selector(advanceSystem) toTarget:self withObject:nil];
//		[NSTimer scheduledTimerWithTimeInterval:0.05 target:self selector:@selector(redrawSystem) userInfo:nil repeats:YES];
    }
	return self;
}

- (void) redrawSystem {
	[self setNeedsDisplay:YES];
}

- (IBAction) setSpeedAction:(id) sender {
	[_system lock];
	_speed = [sender floatValue];
	// NSLog(@"%f", _speed);
	[_system unlock];
}

- (void) advanceSystem {
	NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
	
	while (YES) {
		[_system lock];
//	NSLog(@"advance system");
	// TODO: vary this amount so it is as fine-grained as possible without missing cycles?
		int i;
		float s;
		if (_speed > 0) {
			// TODO: if the user has stopped the universe, stop rotating.
			for (i = 0; i < 100; i++) {
//				[_system advanceTime:60*60*24];	// 1 day
//				[_system advanceTime:60*60*24/3];	// 8 hours
//				[_system advanceTime:60*60];	// 1 hour
				[_system advanceTime:60];	// 1 minute
			}
		}
		s = _speed;
		[_system unlock];
		// NSLog(@"HERE");
		if (s <10000) usleep(10000 - (int)s);
		// NSLog(@"THERE");
		[self redrawSystem];
	}
	[pool drain];
}

- (float) kmToPixels:(float) km {
	return km / _scale;
}

- (float) xPos:(float) kmX {
	return [self frame].size.width / 2 + [self kmToPixels:kmX];
}

- (float) yPos:(float) kmY {
	return [self frame].size.height / 2 + [self kmToPixels:kmY];
}

- (void)drawRect:(NSRect)toDraw {
    // Drawing code here.
	NSBezierPath* path = [NSBezierPath bezierPath];
	NSRect rect;
	
	[[NSColor blackColor] set];
	NSRectFill(toDraw);

	// NSLog(@"drawing");
	[_system lock];
	// draw each star/planet
	NSEnumerator* it = [[_system planets] objectEnumerator];
	Planet *p;
	while (p = (Planet *)[it nextObject]) {
		// TODO: vary width of planets by the mass or radius
		float r;
		// r = [self kmToPixels:[p radius]];
		// r *= 200;
		// r = 3;
		r = log([p radius]);
		rect.origin.x = [self xPos:[p kmX]] - r;
		rect.origin.y = [self yPos:[p kmY]] - r;
		rect.size.width = 2*r;
		rect.size.height = 2*r;
// 		NSLog(@"%@: %0.0f, %0.0f", [p name], rect.origin.x, rect.origin.y);
		// NSLog(@"%f", rect.size.width);
		// NSLog(@"%f", rect.size.height);
		if (NSIntersectsRect(toDraw, rect)) {
			path = [NSBezierPath bezierPathWithOvalInRect:rect];
			[[p color] setFill];
			[path fill];
		}
	}	
	[_system unlock];
}

- (BOOL) isFlipped {
	return YES;
}

- (BOOL) isOpaque {
	return YES;
}

@end
